function [ T1,Y1 ] = ABC_eq_kin(varargin)
%ABC_eq_kin Simulates kinetics of the reaction scheme below via numerical
%   integration of differential rate laws
%   A <--> B <--> C
% Jason Kahn U Maryland 9/27/2016  version 1.01
% Arguments (all optional, but order is fixed and no skipping) in order are
%       A0,B0,C0,k1,k2,k3,k4,t_end,savefig
% Returns a vector of times and an array of []'s of A, B, C
% Makes one plot, a large plain-vanilla concentration vs. time plot
% Typical call with default rate constants and initial concentrations: 
%      [t,Y]= ABC_eq_kin(1,0,0,6,9,2,1,3,0);
%    ABC_eq_kin(1,0,0,60,30,2,1,2.5,0); gives very fast initial
%         equilibration and slow second step
%    ABC_eq_kin(1,0,0,6,4,2,1,5,0);  intermediate behavior
%    ABC_eq_kin(1,0,0,6,0,2,0,3,0);  setting reverse rates = 0 gives
%         classic A->B->C behavior
%    ABC_eq_kin(1,0,0,.6,.4,200,100,5,0); shows first step rate limiting
%         and effecitvely instantaneous equilibration of B and C
%    ABC_eq_kin(1,0,0,.6,.4,1,.5,8,0); illustrates lag in appearance of C
% Calls function ABC_eq_kin_deltas (at the end of this file) to provide differential rates 
% Calls function export_fig if and only if savefig flag is set to 1
%   Download export_fig from http://www.mathworks.com/matlabcentral/fileexchange/23629-exportfig
%
% Define rate constants and initial concentrations
% Defaults:
A0 = 1;
B0 = 0;
C0 = 0;
% D0 = 0;
% E0 = 0;
% F0 = 0;
k1 = 6; % s-1
k2 = 9; % s-1
k3 = 2; % s-1
k4 = 1; % s-1
t_end = 3;
savefig = 0;  % Default is not to write files: savefig = 0
% Set savefig to 1 to activate saving images with exportfig
optargs = {A0,B0,C0,k1,k2,k3,k4,t_end,savefig};
numopts = length(optargs);
% Change default values if user provides them
numvarargs = length(varargin);
if numvarargs > numopts
    error('ABC_eq_kin:TooManyInputs', ...
        'ABC_eq_kin allows at most %d optional inputs',numopts);
end

% now put the new values from varargin into the optargs cell array, 
% overwriting the default ones
optargs(1:numvarargs) = varargin;

% Place optional args back into into memorable variable names
[A0,B0,C0,k1,k2,k3,k4,t_end,savefig] = optargs{:};
concs0 = [A0,B0,C0];
rateks = [k1,k2,k3,k4];
str = {['k1 = ',num2str(k1),' s-1'],['k2 = ',num2str(k2), ' s-1'],['k3 = ',num2str(k3), ' s-1'],...
    ['k4 = ',num2str(k4), ' s-1'],['End time is ',num2str(t_end), ' s'],...
    ['Save figure flag is ',num2str(savefig)]};
str2 = {['A0 = ',num2str(A0),' M'],['B0 = ',num2str(B0), ' M'],['C0 = ',num2str(C0), ' M']};
str3 = {['A <=> B, k1 and k2 '],['B <=> C, k3 and k4']};
% pnum is for numbering plots sequentially. 
% Limit number of files exported to lim_files
% savefig = 1 means we are saving files
if savefig == 1;
    lim_files = 50;
    fileout = 'ABC_eq_kin';
    pnum = 1;
    while (pnum <= lim_files)
        fntest = [fileout,num2str(pnum),'.jpg'];
        if (exist(fntest,'file'))
            pnum = pnum+1;
        else
            disp(['Exported file is ',fntest])
            break
        end
    end
    if (pnum == lim_files+1)
        disp(['Exceeded file count: overwriting ',fntest])
    end
end

% Integrate differential equation to give time course. This is the
% point of the whole exercise. The ode15s solver is better for stiff differential
% equations.
options = odeset('RelTol',1e-6,'AbsTol',1e-4,'NonNegative',[1 2 3]);
[T1,Y1] = ode15s(@(t,concs) ABC_eq_kin_deltas(t,concs,rateks),[0 t_end],concs0',options);
% First a plain vanilla plot of the results:
figure1 = figure('Position',[10 10 1024 768]);
plot(T1,Y1,'LineWidth',2)
title(['ABC eq Kinetics'],'FontSize',16)
xlabel('Time','FontSize',14)
ylabel('Concentrations','FontSize',14)
legend({'A','B','C'},'FontSize',12)
%

annotation(figure1,'textbox', [.2 .75 .135 .140],'BackGroundColor',[1 1 1],...
           'String', str2,'FitBoxToText','on','FontSize',14);
annotation(figure1,'textbox', [.6 .75 .135 .130],'BackGroundColor',[1 1 1],...
           'String', str,'FitBoxToText','on','FontSize',14);
annotation(figure1,'textbox', [.7 .55 .175 .130],'BackGroundColor',[1 1 1],...
           'String', str3,'FitBoxToText','on','FontSize',14);
if (savefig == 1)
    export_fig (figure1,fntest)
end
end

function dconcs = ABC_eq_kin_deltas(~,concs,rateks)
% Calculate the change in concentration of species in concs array
% return the change in each concentration
% These deltas are integrated in the calling program. 
dconcs = zeros(length(concs),1);
dconcs(1) = -rateks(1)*concs(1) + rateks(2)*concs(2);
dconcs(2) = rateks(1)*concs(1) - rateks(2)*concs(2) - rateks(3)*concs(2) + rateks(4)*concs(3);
dconcs(3) = rateks(3)*concs(2) - rateks(4)*concs(3);
end

